/* Sample from Multivariate Normal distribution with (Mean, Cov).
N		is the number of desired observations sampled from the
		multivariate normal distribution.
Mean	is a 1xp vector of means.
Cov		is a pxp symmetric positive definite variance-covariance matrix.

Each row of the returned matrix is a row vector
sampled from the multivariate normal distribution.
*/

proc iml;
start RANDNORMAL( N, Mean , Cov );
	/* Algorithm: 
	1. By a Cholesky factorization, get T such that T`T = variance.
	2. Generate a p-vector of iid N(0,1) random variables z = (z_1, ... ,z_p).
	3. x = T`z + mean. 
	4. Then x follows the multivariate normal distribution with Mean, Cov. 

	Reference: 
	Gentle, J.E. (2003), Random Number Generation and Monte Carlo Methods, 
		New York: Springer-Verlag, Inc., 197-198.
*/
	/* check parameters */
	if N<1 then do;
	print "The requested number of observations should be at least 1:" N; stop;
	end;

	mMean = rowvec(Mean);
	p = ncol(mMean); 
	/* Upper triangular matrix T from the Cholesky decomposition: 
		variance = symmetric positive-definite matrix.
	   This call will fail if the matrix is not sym. pos. def. */
	T = root( Cov ); 
	Z=j(N,p,.);
	call randgen(Z,'NORMAL');
	outX = Z*T + repeat(mMean,N,1);
	return(outX);
finish;
store module=RANDNORMAL;
quit;

